//
//  BTFCoreDataManager.m
//  BackgroundTableFetcher
//
//  Created by Krzysztof Cieplucha on 17/11/13.
//  Copyright (c) 2013 Krzysztof Cieplucha. All rights reserved.
//

#import "BTFCoreDataManager.h"

#import "BTFUser.h"

@implementation BTFCoreDataManager

#pragma mark - class methods

+ (BTFCoreDataManager *)sharedInstance {
    static BTFCoreDataManager *sharedInstance = nil;
    static dispatch_once_t onceToken;
    dispatch_once(&onceToken, ^{
        sharedInstance = [[BTFCoreDataManager alloc] init];
    });
    return sharedInstance;
}

#pragma mark - initializers

- (id)init {
    self = [super init];
    if (self) {
        _managedObjectContext = [self _createManagedObjectContext];
        
        [[NSNotificationCenter defaultCenter] addObserver:self selector:@selector(_contextDidSave:) name:NSManagedObjectContextDidSaveNotification object:nil];
    }
    return self;
}

- (void)dealloc {
    [[NSNotificationCenter defaultCenter] removeObserver:self];
}

#pragma mark - properties

#pragma mark - instance methods

- (NSFetchedResultsController *)createFetchResultsController {
    NSFetchRequest *fetchRequest = [NSFetchRequest fetchRequestWithEntityName:@"User"];
    fetchRequest.sortDescriptors = @[[NSSortDescriptor sortDescriptorWithKey:@"name" ascending:YES]];
    fetchRequest.fetchBatchSize = 100;
    return [[NSFetchedResultsController alloc] initWithFetchRequest:fetchRequest managedObjectContext:_managedObjectContext sectionNameKeyPath:@"sectionName" cacheName:nil];
}

- (void)modifyData {
    NSManagedObjectContext *managedObjectContext = [self _createManagedObjectContext];
    [managedObjectContext performBlockAndWait:^{
        NSFetchRequest *fetchRequest = [NSFetchRequest fetchRequestWithEntityName:@"User"];
        fetchRequest.sortDescriptors = @[[NSSortDescriptor sortDescriptorWithKey:@"name" ascending:YES]];
        NSArray *objects = [managedObjectContext executeFetchRequest:fetchRequest error:nil];
        
        for (BTFUser *user in objects) {
            user.identifier = [NSUUID new].UUIDString;
            if (managedObjectContext.updatedObjects.count >= 1000) {
                [managedObjectContext save:nil];
            }
        }
        [managedObjectContext save:nil];
    }];
}

- (void)prepareData {
    NSManagedObjectContext *managedObjectContext = [self _createManagedObjectContext];
    [managedObjectContext performBlockAndWait:^{
        NSFetchRequest *fetchRequest = [NSFetchRequest fetchRequestWithEntityName:@"User"];
        NSUInteger count = [managedObjectContext countForFetchRequest:fetchRequest error:nil];
        
        if (!count) {
            for (int i = 0; i < 100000; ++i) {
                BTFUser *user = [NSEntityDescription insertNewObjectForEntityForName:@"User" inManagedObjectContext:managedObjectContext];
                user.name = [NSUUID new].UUIDString;
                user.identifier = [NSUUID new].UUIDString;
                user.sectionName = [user.name substringToIndex:1];
            }
            [managedObjectContext save:nil];
        }
    }];
}

#pragma mark - inherited methods

#pragma mark - private methods

- (void)_contextDidSave:(NSNotification *)notification {
    [_managedObjectContext performBlockAndWait:^{
        [_managedObjectContext mergeChangesFromContextDidSaveNotification:notification];
        [_managedObjectContext processPendingChanges];
    }];
}

- (NSManagedObjectContext *)_createManagedObjectContext {
    NSManagedObjectContext *managedObjectContext = [[NSManagedObjectContext alloc] initWithConcurrencyType:NSPrivateQueueConcurrencyType];
    managedObjectContext.persistentStoreCoordinator = [self _createPersistentStoreCoordinator];
    return managedObjectContext;
}

- (NSPersistentStoreCoordinator *)_createPersistentStoreCoordinator {
    NSManagedObjectModel *managedObjectModel = [NSManagedObjectModel mergedModelFromBundles:nil];
    NSPersistentStoreCoordinator *persistentStoreCoordinator = [[NSPersistentStoreCoordinator alloc] initWithManagedObjectModel:managedObjectModel];
    NSURL *persistentStoreURL = [[[NSFileManager defaultManager] URLsForDirectory:NSDocumentDirectory inDomains:NSUserDomainMask].firstObject URLByAppendingPathComponent:@"store.sqlite"];
    [persistentStoreCoordinator addPersistentStoreWithType:NSSQLiteStoreType configuration:nil URL:persistentStoreURL options:nil error:nil];
    return persistentStoreCoordinator;
}

#pragma mark - interface implementation

@end
