//
//  BTFMainViewController.m
//  BackgroundTableFetcher
//
//  Created by Krzysztof Cieplucha on 17/11/13.
//  Copyright (c) 2013 Krzysztof Cieplucha. All rights reserved.
//

#import "BTFMainViewController.h"

#import "BTFCoreDataManager.h"
#import "BTFUser.h"

@implementation BTFMainViewController

#pragma mark - class methods

#pragma mark - initializers

- (id)init {
    self = [super init];
    if (self) {
        _modifyDataBarButtonItem = [[UIBarButtonItem alloc] initWithBarButtonSystemItem:UIBarButtonSystemItemRefresh target:self action:@selector(_modifyDataButtonPressed:)];
        _fetchedResultsController = [[BTFCoreDataManager sharedInstance] createFetchResultsController];
        _fetchedResultsController.delegate = self;
    }
    return self;
}

#pragma mark - properties

#pragma mark - instance methods

#pragma mark - inherited methods

- (void)viewDidLoad {
    [super viewDidLoad];
    
    self.navigationItem.rightBarButtonItem = _modifyDataBarButtonItem;
    self.navigationItem.title = NSLocalizedString(@"Loading...", nil);
    [[BTFCoreDataManager sharedInstance].managedObjectContext performBlock:^{
        [_fetchedResultsController performFetch:nil];
        dispatch_async(dispatch_get_main_queue(), ^{
            [self.tableView reloadData];
            self.navigationItem.title = NSLocalizedString(@"Data", nil);
        });
    }];
}

#pragma mark - private methods

- (void)_modifyDataButtonPressed:(id)sender {
    [sender setEnabled:NO];
    self.navigationItem.title = NSLocalizedString(@"Changeing...", nil);
    dispatch_async(dispatch_get_global_queue(DISPATCH_QUEUE_PRIORITY_DEFAULT, 0), ^{
        [[BTFCoreDataManager sharedInstance] modifyData];
        dispatch_async(dispatch_get_main_queue(), ^{
            self.navigationItem.title = NSLocalizedString(@"Data", nil);
            [sender setEnabled:YES];
        });
    });
}

#pragma mark - interface implementation

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
    return _fetchedResultsController.sections.count;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    return [_fetchedResultsController.sections[section] numberOfObjects];
}

- (NSString *)tableView:(UITableView *)tableView titleForHeaderInSection:(NSInteger)section {
    return [_fetchedResultsController.sections[section] name];
}

- (NSArray *)sectionIndexTitlesForTableView:(UITableView *)tableView {
    return [_fetchedResultsController sectionIndexTitles];
}

- (NSInteger)tableView:(UITableView *)tableView sectionForSectionIndexTitle:(NSString *)title atIndex:(NSInteger)index {
    return [_fetchedResultsController sectionForSectionIndexTitle:title atIndex:index];
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    static NSString *cellIdentifier = @"cellIdentifier";
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:cellIdentifier];
    if (!cell) {
        cell = [[UITableViewCell alloc] initWithStyle:UITableViewCellStyleSubtitle reuseIdentifier:cellIdentifier];
    }
    
    __block NSString *text = nil;
    __block NSString *detailText = nil;
    [[BTFCoreDataManager sharedInstance].managedObjectContext performBlockAndWait:^{
        BTFUser *user = [_fetchedResultsController objectAtIndexPath:indexPath];
        text = user.name;
        detailText = user.identifier;
    }];
    cell.textLabel.text = text;
    cell.detailTextLabel.text = detailText;
    
    return cell;
}

- (void)controllerDidChangeContent:(NSFetchedResultsController *)controller {
    dispatch_async(dispatch_get_main_queue(), ^{
        [self.tableView reloadData];
    });
}

@end
